<?php

/**
 * The control file of ktdevice module
 *
 * @copyright   Shanghai Kuntai Software Technology Co., Ltd. 2022
 * @author      kuntai
 * @package     ktdevice
 */
class ktdevice extends control {

	/**
	 * Construct function.
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {
		parent::__construct();
		$this->app->loadClass('date');
		$this->loadModel('dept');
		$this->loadModel('ktbase');
	}

	/**
	 * Copy a ktdevice.
	 *
	 * @param int $groupID
	 * @access public
	 * @return void
	 */
	public function groupcopy($groupID) {
		if (!empty($_POST)) {
			$this->ktdevice->groupcopy($groupID);
			if (dao::isError()) die(js::error(dao::getError()));
			if (isonlybody()) die(js::closeModal('parent.parent', 'this'));
			die(js::locate($this->createLink('ktdevice', 'browse'), 'parent'));
		}

		$this->view->title      = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->groupcopy;
		$this->view->position[] = $this->lang->ktdevice->groupcopy;
		$this->view->group      = $this->ktdevice->groupGetById($groupID);
		$this->view->subMenu    = "grouplist";
		$this->display();
	}

	/**
	 * Create a ktdevice.
	 *
	 * @access public
	 * @return void
	 */
	public function groupcreate() {
		if (!empty($_POST)) {
			$this->ktdevice->groupcreate();
			if (dao::isError()) die(js::error(dao::getError()));
			if (isonlybody()) die(js::closeModal('parent.parent', 'this'));
			die(js::locate($this->createLink('ktdevice', 'grouplist'), 'parent'));
		}

		$this->view->title      = $this->lang->ktdevice->title.$this->lang->colon.$this->lang->ktdevice->groupcreate;
		$this->view->position[] = $this->lang->ktdevice->groupcreate;
		$this->view->subMenu    = "grouplist";
		$this->display();
	}

	/**
	 * Delete a ktdevice.
	 *
	 * @param int    $groupID
	 * @param string $confirm yes|no
	 * @access public
	 * @return void
	 */
	public function groupdelete($groupID, $confirm = 'no') {
		if ($confirm == 'no') {
			die(js::confirm($this->lang->ktdevice->confirmDelete.$this->lang->ktdevice->group."?", $this->createLink('ktdevice', 'groupdelete', "groupID=$groupID&confirm=yes"))
				.js::closeModal('parent.parent', 'this'));
		} else {
			$this->ktdevice->groupdelete($groupID);
			if (dao::isError()) die(js::error(dao::getError()));
			if (isonlybody()) die(js::closeModal('parent.parent', 'this'));
			die(js::locate($this->createLink('ktdevice', 'grouplist'), 'parent'));
		}
	}

	/**
	 * Edit a ktdevice.
	 *
	 * @param int $groupID
	 * @access public
	 * @return void
	 */
	public function groupedit($groupID) {
		if (!empty($_POST)) {
			$this->ktdevice->groupUpdate($groupID);
			if (isonlybody()) die(js::closeModal('parent.parent', 'this'));
			die(js::locate($this->createLink('ktdevice', 'browse'), 'parent'));
		}

		$title                = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->groupedit;
		$position[]           = $this->lang->ktdevice->groupedit;
		$this->view->title    = $title;
		$this->view->position = $position;
		$this->view->group    = $this->ktdevice->groupGetById($groupID);
		$this->view->subMenu  = "grouplist";

		$this->display();
	}

	/**
	 * Browse groups.
	 *
	 * @param int $companyID
	 * @access public
	 * @return void
	 */
	public function grouplist() {
		$title      = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->grouplist;
		$position[] = $this->lang->ktdevice->grouplist;

		$groups     = $this->ktdevice->groupGetList();
		$groupUsers = array();
		foreach ($groups as $group)
			$groupUsers[$group->id] = $this->ktdevice->devGetPairs($group->id);

		$this->view->title             = $title;
		$this->view->position          = $position;
		$this->view->groups            = $groups;
		$this->view->groupUsers        = $groupUsers;
		$this->view->subMenu           = "grouplist";
		$this->lang->modulePageActions = html::a(helper::createLink('ktdevice', 'groupcreate', '', '', true), "<i class='icon-plus'></i> ".$this->lang->ktdevice->groupcreate, '', "class='btn btn-primary iframe'");

		$this->display();
	}

	/**
	 * view a group.
	 *
	 * @param int $groupID the int group id
	 * @access public
	 * @return void
	 */
	public function groupview($groupID) {
		if (!$groupID) {
			die(js::error("Input group ID wrong, must a right number!"));
		}
		$this->view->group = $this->ktdevice->groupGetById($groupID);

		if (!$this->view->group) {
			die(js::error("Can not find the group by this id = ".$groupID));
		}

		$title                = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->groupedit;
		$position[]           = $this->lang->ktdevice->grouplist;
		$this->view->title    = $title;
		$this->view->devices  = $this->ktdevice->devGetByGroup($groupID);
		$this->view->position = $position;

		$this->view->subMenu = "grouplist";
		$this->display();
	}

	/**
	 * Create a suer.
	 *
	 * @param int $deptID
	 * @access public
	 * @return void
	 */
	public function devcreate($deptID = 0) {
		if (!empty($_POST)) {
			$this->ktdevice->devcreate();

			if (dao::isError()) die(js::error(dao::getError()));
			if (isonlybody()) die(js::closeModal('parent.parent', 'this'));
			die(js::locate($this->createLink('ktdevice', 'devlist'), 'parent'));
		}
		$groups    = $this->dao->select('id, name, type')->from(TABLE_KT_DEVICE_GROUP)->fetchAll();
		$groupList = array('' => '');
		$typeGroup = array();
		foreach ($groups as $group) {
			$groupList[$group->id] = $group->name;
			if ($group->type) $typeGroup[$group->type] = $group->id;
		}

		$title                = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->devlist;
		$position[]           = $this->lang->ktdevice->common;
		$position[]           = $this->lang->ktdevice->devlist;
		$this->view->title    = $title;
		$this->view->position = $position;

		$this->view->depts     = $this->dept->getOptionMenu();
		$this->view->groupList = $groupList;
		$this->view->typeGroup = $typeGroup;
		$this->view->deptID    = $deptID;
		$this->view->subMenu   = "devlist";

		$this->display();
	}

	/**
	 * Delete a device.
	 *
	 * @param int    $deviceID
	 * @param string $confirm yes|no
	 * @access public
	 * @return void
	 */
	public function devdelete($deviceID, $confirm = 'no') {
		if (!$deviceID) return;
		if ($confirm == 'no') {
			die(js::confirm($this->lang->ktdevice->confirmDelete.$this->lang->ktdevice->dev
					, $this->createLink('ktdevice', 'devdelete', "userID=$deviceID&confirm=yes"))
				.js::closeModal('parent.parent', 'this'));
		} else {
			$this->ktdevice->devdelete($deviceID);
			if (dao::isError()) die(js::error(dao::getError()));
			if (isonlybody()) die(js::closeModal('parent.parent', 'this'));
			die(js::locate($this->createLink('ktdevice', 'devlist'), 'parent'));
		}
	}

	/**
	 * Edit a device.
	 *
	 * @param int $deviceID the int device id
	 * @access public
	 * @return void
	 */
	public function devedit($deviceID) {
		if (!empty($_POST)) {
			$this->ktdevice->devupdate($deviceID);
			if (dao::isError()) die(js::error(dao::getError()));
			if (isonlybody()) die(js::closeModal('parent.parent', 'this'));
			die(js::locate($this->createLink('ktdevice', 'devlist'), 'parent'));
		}
		$this->loadModel("user");
		$device = $this->ktdevice->devGetById($deviceID);
		if (!$device) die(js::error("Can not Get Device with id = ".$deviceID));

		$this->view->users = $this->user->getPairs('nodeleted');
		$users             = &$this->view->users;
		if (!array_key_exists($device->user, $users)) $users[$device->user] = $device->user;
		if (!array_key_exists($device->charge, $users)) $users[$device->charge] = $device->user;

		$deviceGroups = $this->ktdevice->groupGetByDevice($device->id);

		$title                    = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->devedit;
		$position[]               = $this->lang->ktdevice->devlist;
		$this->view->title        = $title;
		$this->view->position     = $position;
		$this->view->device       = $device;
		$this->view->depts        = $this->dept->getOptionMenu();
		$this->view->deviceGroups = implode(',', array_keys($deviceGroups));
		$this->view->groups       = $this->ktdevice->groupGetPairs();
		$this->view->subMenu      = "devlist";

		$this->display();
	}

	/**
	 * Browse group and devices.
	 *
	 * @param int    $param
	 * @param string $type
	 * @param string $orderBy
	 * @param int    $recTotal
	 * @param int    $recPerPage
	 * @param int    $pageID
	 * @access public
	 * @return void
	 */
	public function devlist($param = 0, $type = 'bygroup', $orderBy = 'id', $recTotal = 0, $recPerPage = 20, $pageID = 1) {
		$groupID = $type == 'bygroup' ? (int)$param : 0;

		/* Set the pager. */
		$this->app->loadClass('pager', $static = true);
		$pager = pager::init($recTotal, $recPerPage, $pageID);

		$sort = $this->ktbase->appendOrder($orderBy);

		$kt_showStyle = 1; //默认简单
		//if post ,delete filter key word
		if (!empty($_POST)) {
			$kt_showStyle = $this->post->showStyle;
			if ($kt_showStyle) {
				$this->session->set("kt_showStyle", $kt_showStyle); //save sesison
			}
		} else {
			if (isset($_SESSION['kt_showStyle'])) {
				$kt_showStyle = $_SESSION['kt_showStyle']; //获得session中保存的筛选关键词
			}
		}

		$this->loadModel("user");
		$this->view->users     = $this->user->getPairs('noletter');
		$this->view->showStyle = $kt_showStyle;

		//$userQuery = ;
		$devices                 = $this->ktdevice->devGetByQuery($groupID, $pager, $sort);
		$this->view->targetgroup = ($groupID) ? $this->ktdevice->groupGetById($groupID) : 0;

		$groups                 = $this->ktdevice->groupGetList();
		$this->view->title      = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->devlist;
		$this->view->position[] = $this->lang->ktdevice->common;
		$this->view->devices    = $devices;
		$this->view->orderBy    = $orderBy;
		$this->view->groupID    = $groupID;
		$this->view->pager      = $pager;
		$this->view->param      = $param;
		$this->view->groups     = $groups;
		$this->view->type       = $type;
		$this->view->subMenu    = "devlist";

		$this->display();
	}

	public function devxlsexport($groupID = 0, $orderBy = 'id') {
		$devices = $this->ktdevice->devGetByQuery($groupID, null, $orderBy);
		$this->loadModel("user");
		$users      = $this->user->getPairs();
		$configRoot = dirname(__FILE__).DIRECTORY_SEPARATOR.'../../lib';
		require_once $configRoot.'/phpexcel/PHPExcel.php';
		require_once $configRoot.'/phpexcel/PHPExcel/Writer/Excel5.php';

		$randname = $this->ktdevice->createRandomStr(10);
		$filename = '..\..\www\data\tmp\\'.$randname.'.xls';
		if (!file_exists('..\..\www\data\tmp')) @mkdir('..\..\www\data\tmp');

		$handle      = fopen($filename, 'w');
		$objPHPExcel = \PHPExcel_IOFactory::load($filename);

		$titles =& $this->config->ktdevice->titles;
		$objPHPExcel->setActiveSheetIndex(0);
		for ($i = 0; $i <= count($devices); $i++) {
			$col = 'A';
			foreach ($titles as $row => $title) {
				if ($i === 0) {
					if ($title == 'id') $thead = 'ID';
					else $thead = $this->lang->ktdevice->$title;
					$objPHPExcel->getActiveSheet()->setCellValue(($col++).($i + 1), $thead);
				} else {
					$dev =& $devices[$i - 1];
					$objPHPExcel->getActiveSheet()->setCellValue(($col++).($i + 1), $this->ktdevice->devalchk($dev, $title, $users));
				}
			}
		}
		fclose($handle);
		ob_end_clean();
		header('Content-type:application/vnd.ms-excel;charset=UTF-8BOM');
		header('Content-Disposition:attachment;filename="'.$this->lang->ktdevice->devxlsexport.'.xls'.'"');
		header("Pragma:nono-cache");
		header("Expires:0");
		$objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel5');
		$objWriter->save('php://output');
		@unlink($filename);
	}

	/**
	 * view a device.
	 *
	 * @param int $deviceID the int device id
	 * @access public
	 * @return void
	 */
	public function devview($deviceID) {
		if (!$deviceID) die(js::error("Input device ID wrong, must a right number!"));

		$device = $this->ktdevice->devGetById($deviceID);
		if (!$device) die(js::error("Can not find the device with id = ".$deviceID));

		$deviceGroups = $this->ktdevice->groupGetByDevice($device->id);

		$dept = $this->dept->getByID($device->dept);

		$this->loadModel("user");
		$users = $this->user->getPairs('nodeleted');
		if (!array_key_exists($device->user, $users)) $users[$device->user] = $device->user;
		if (!array_key_exists($device->charge, $users)) $users[$device->charge] = $device->user;

		if ($device) {
			$device->userName   = $users[$device->user];
			$device->chargeName = $users[$device->charge];
		} else {
			$device->chargeName = "";
			$device->userName   = "";
		}

		$title                    = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->devview;
		$position[]               = $this->lang->ktdevice->devlist;
		$this->view->title        = $title;
		$this->view->position     = $position;
		$this->view->device       = $device;
		$this->view->deptName     = (!$dept) ? "" : $dept->name;
		$this->view->deviceGroups = $deviceGroups;
		$this->view->subMenu      = "devlist";


		$this->display();
	}

	/**
	 * Statistic ktdevices.
	 * @param string $date
	 * @param string $type
	 * @access public
	 * @return void
	 */
	public function statistic($type = 'group', $year = 0) {
		//网页位置
		$this->view->title      = $this->lang->ktdevice->common.$this->lang->colon.$this->lang->ktdevice->statistic;
		$this->view->position[] = $this->lang->ktdevice->statistic;

		//绘制图表
		if ('group' == $type) {
			$this->view->items = $this->ktdevice->statisticByGroup();
		} else if ('dept' == $type) {
			$this->view->items = $this->ktdevice->statisticByDept();
		} else if ('type' == $type) {
			$this->view->items = $this->ktdevice->statisticByType();
		} else if ('yearlimit' == $type) {
			$this->view->items = $this->ktdevice->statisticByYear();
        } else {
			$this->view->ErrorMsg = 'input type is wrong. no suche type of "'.$type.'"';
		}
		//页面参数
		$this->view->statisticType = $type;
		$this->view->subMenu       = "statistic";
		$this->display();
	}

}
